@@ -16,14 +16,13 @@
 #include "kdeconnect-version.h"
 #include <KAboutData>
 #include <KColorSchemeManager>
+#include <KCrash>
 #include <KLocalizedContext>
 #include <KLocalizedString>
 
 int main(int argc, char *argv[])
 {
     QIcon::setFallbackThemeName(QStringLiteral("breeze"));
-    QGuiApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
-    QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
 
     QApplication app(argc, argv);
     KLocalizedString::setApplicationDomain("kdeconnect-app");
@@ -36,10 +35,13 @@ int main(int argc, char *argv[])
                          i18n("(c) 2015, Aleix Pol Gonzalez"));
     aboutData.addAuthor(i18n("Aleix Pol Gonzalez"), i18n("Maintainer"), QStringLiteral("aleixpol@kde.org"));
     aboutData.setTranslator(i18nc("NAME OF TRANSLATORS", "Your names"), i18nc("EMAIL OF TRANSLATORS", "Your emails"));
+    aboutData.setBugAddress("https://bugs.kde.org/enter_bug.cgi?product=kdeconnect&component=common");
     KAboutData::setApplicationData(aboutData);
 
+    KCrash::initialize();
+
 #ifdef Q_OS_WIN
-    KColorSchemeManager manager;
+    KColorSchemeManager::instance();
     QApplication::setStyle(QStringLiteral("breeze"));
 #endif
 
@@ -72,7 +74,7 @@ int main(int argc, char *argv[])
 
     QQmlApplicationEngine engine;
     engine.rootContext()->setContextObject(new KLocalizedContext(&engine));
-    engine.load(QUrl(QStringLiteral("qrc:/qml/main.qml")));
+    engine.loadFromModule("org.kde.kdeconnect.app", "Main");
 
     return app.exec();
 }
