@@ -12,6 +12,7 @@
 #include <QTextStream>
 
 #include <KAboutData>
+#include <KCrash>
 
 #include "interfaces/conversationmessage.h"
 #include "interfaces/dbushelpers.h"
@@ -33,12 +34,14 @@ int main(int argc, char **argv)
                      i18n("(C) 2015 Aleix Pol Gonzalez"));
     KAboutData::setApplicationData(about);
 
+    KCrash::initialize();
+
     about.addAuthor(i18n("Aleix Pol Gonzalez"), QString(), QStringLiteral("aleixpol@kde.org"));
     about.addAuthor(i18n("Albert Vaca Cintora"), QString(), QStringLiteral("albertvaka@gmail.com"));
     QCommandLineParser parser;
-    parser.addOption(QCommandLineOption(QStringList(QStringLiteral("l")) << QStringLiteral("list-devices"), i18n("List all devices")));
+    parser.addOption(QCommandLineOption(QStringList{QStringLiteral("l"), QStringLiteral("list-devices")}, i18n("List all devices")));
     parser.addOption(
-        QCommandLineOption(QStringList(QStringLiteral("a")) << QStringLiteral("list-available"), i18n("List available (paired and reachable) devices")));
+        QCommandLineOption(QStringList{QStringLiteral("a"), QStringLiteral("list-available")}, i18n("List available (paired and reachable) devices")));
     parser.addOption(
         QCommandLineOption(QStringLiteral("id-only"), i18n("Make --list-devices or --list-available print only the devices id, to ease scripting")));
     parser.addOption(
@@ -62,15 +65,14 @@ int main(int argc, char **argv)
     parser.addOption(QCommandLineOption(QStringLiteral("attachment"),
                                         i18n("File urls to send attachments with the message (can be passed multiple times)"),
                                         i18n("file urls")));
-    parser.addOption(QCommandLineOption(QStringList(QStringLiteral("device")) << QStringLiteral("d"), i18n("Device ID"), QStringLiteral("dev")));
-    parser.addOption(QCommandLineOption(QStringList(QStringLiteral("name")) << QStringLiteral("n"), i18n("Device Name"), QStringLiteral("name")));
+    parser.addOption(QCommandLineOption(QStringList{QStringLiteral("device"), QStringLiteral("d")}, i18n("Device ID"), QStringLiteral("dev")));
+    parser.addOption(QCommandLineOption(QStringList{QStringLiteral("name"), QStringLiteral("n")}, i18n("Device Name"), QStringLiteral("name")));
     parser.addOption(QCommandLineOption(QStringLiteral("encryption-info"), i18n("Get encryption info about said device")));
     parser.addOption(QCommandLineOption(QStringLiteral("list-commands"), i18n("Lists remote commands and their ids")));
     parser.addOption(QCommandLineOption(QStringLiteral("execute-command"), i18n("Executes a remote command by id"), QStringLiteral("id")));
     parser.addOption(
         QCommandLineOption(QStringList{QStringLiteral("k"), QStringLiteral("send-keys")}, i18n("Sends keys to a said device"), QStringLiteral("key")));
     parser.addOption(QCommandLineOption(QStringLiteral("my-id"), i18n("Display this device's id and exit")));
-    parser.addOption(QCommandLineOption(QStringLiteral("photo"), i18n("Open the connected device's camera and transfer the photo"), QStringLiteral("path")));
 
     // Hidden because it's an implementation detail
     QCommandLineOption deviceAutocomplete(QStringLiteral("shell-device-autocompletion"));
@@ -84,7 +86,6 @@ int main(int argc, char **argv)
     parser.process(app);
     about.processCommandLine(&parser);
 
-    const QString id = QStringLiteral("kdeconnect-cli-") + QString::number(QCoreApplication::applicationPid());
     DaemonDbusInterface iface;
 
     if (parser.isSet(QStringLiteral("my-id"))) {
@@ -206,22 +207,22 @@ int main(int argc, char **argv)
             }
 
             QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"),
-                                                              QStringLiteral("/modules/kdeconnect/devices/") + device + QStringLiteral("/share"),
+                                                              QLatin1String("/modules/kdeconnect/devices/%1/share").arg(device),
                                                               QStringLiteral("org.kde.kdeconnect.device.share"),
                                                               QStringLiteral("shareUrls"));
 
-            msg.setArguments(QVariantList() << QVariant(urls));
+            msg.setArguments(QVariantList{QVariant(urls)});
             blockOnReply(QDBusConnection::sessionBus().asyncCall(msg));
 
-            for (const QString &url : qAsConst(urls)) {
+            for (const QString &url : std::as_const(urls)) {
                 QTextStream(stdout) << i18n("Shared %1", url) << Qt::endl;
             }
         } else if (parser.isSet(QStringLiteral("share-text"))) {
             QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"),
-                                                              QStringLiteral("/modules/kdeconnect/devices/") + device + QStringLiteral("/share"),
+                                                              QLatin1String("/modules/kdeconnect/devices/%1/share").arg(device),
                                                               QStringLiteral("org.kde.kdeconnect.device.share"),
                                                               QStringLiteral("shareText"));
-            msg.setArguments(QVariantList() << parser.value(QStringLiteral("share-text")));
+            msg.setArguments(QVariantList{parser.value(QStringLiteral("share-text"))});
             blockOnReply(QDBusConnection::sessionBus().asyncCall(msg));
             QTextStream(stdout) << i18n("Shared text: %1", parser.value(QStringLiteral("share-text"))) << Qt::endl;
         } else if (parser.isSet(QStringLiteral("lock")) || parser.isSet(QStringLiteral("unlock"))) {
@@ -269,18 +270,18 @@ int main(int argc, char **argv)
             }
         } else if (parser.isSet(QStringLiteral("send-clipboard"))) {
             QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"),
-                                                              QStringLiteral("/modules/kdeconnect/devices/") + device + QStringLiteral("/clipboard"),
+                                                              QLatin1String("/modules/kdeconnect/devices/%1/clipboard").arg(device),
                                                               QStringLiteral("org.kde.kdeconnect.device.clipboard"),
                                                               QStringLiteral("sendClipboard"));
             blockOnReply(QDBusConnection::sessionBus().asyncCall(msg));
         } else if (parser.isSet(QStringLiteral("ping")) || parser.isSet(QStringLiteral("ping-msg"))) {
             QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"),
-                                                              QStringLiteral("/modules/kdeconnect/devices/") + device + QStringLiteral("/ping"),
+                                                              QLatin1String("/modules/kdeconnect/devices/%1/ping").arg(device),
                                                               QStringLiteral("org.kde.kdeconnect.device.ping"),
                                                               QStringLiteral("sendPing"));
             if (parser.isSet(QStringLiteral("ping-msg"))) {
                 QString message = parser.value(QStringLiteral("ping-msg"));
-                msg.setArguments(QVariantList() << message);
+                msg.setArguments(QVariantList{message});
             }
             blockOnReply(QDBusConnection::sessionBus().asyncCall(msg));
         } else if (parser.isSet(QStringLiteral("send-sms"))) {
@@ -315,26 +316,14 @@ int main(int argc, char **argv)
             }
         } else if (parser.isSet(QStringLiteral("ring"))) {
             QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"),
-                                                              QStringLiteral("/modules/kdeconnect/devices/") + device + QStringLiteral("/findmyphone"),
+                                                              QLatin1String("/modules/kdeconnect/devices/%1/findmyphone").arg(device),
                                                               QStringLiteral("org.kde.kdeconnect.device.findmyphone"),
                                                               QStringLiteral("ring"));
             blockOnReply(QDBusConnection::sessionBus().asyncCall(msg));
-        } else if (parser.isSet(QStringLiteral("photo"))) {
-            const QString fileName = parser.value(QStringLiteral("photo"));
-            if (!fileName.isEmpty()) {
-                QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"),
-                                                                  QStringLiteral("/modules/kdeconnect/devices/") + device + QStringLiteral("/photo"),
-                                                                  QStringLiteral("org.kde.kdeconnect.device.photo"),
-                                                                  QStringLiteral("requestPhoto"));
-                msg.setArguments({QUrl::fromLocalFile(fileName).toString()});
-                blockOnReply(QDBusConnection::sessionBus().asyncCall(msg));
-            } else {
-                QTextStream(stderr) << i18n("Please specify a filename for the photo") << Qt::endl;
-            }
         } else if (parser.isSet(QStringLiteral("send-keys"))) {
             QString seq = parser.value(QStringLiteral("send-keys"));
             QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"),
-                                                              QStringLiteral("/modules/kdeconnect/devices/") + device + QStringLiteral("/remotekeyboard"),
+                                                              QLatin1String("/modules/kdeconnect/devices/%1/remotekeyboard").arg(device),
                                                               QStringLiteral("org.kde.kdeconnect.device.remotekeyboard"),
                                                               QStringLiteral("sendKeyPress"));
             if (seq.trimmed() == QLatin1String("-")) {
