@@ -12,13 +12,19 @@
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 #include <sys/socket.h>
+#else
+#include <winsock2.h>
+// Winsock2 needs to be included before any other header
+#include <mstcpip.h>
+#endif
+
+#if defined(Q_OS_WIN) || defined(Q_OS_FREEBSD)
+#include <QNetworkInterface>
 #endif
 
 #include <QHostInfo>
 #include <QMetaEnum>
-#include <QNetworkConfigurationManager>
 #include <QNetworkProxy>
-#include <QNetworkSession>
 #include <QSslCipher>
 #include <QSslConfiguration>
 #include <QSslKey>
@@ -27,28 +33,28 @@
 #include <QUdpSocket>
 
 #include "daemon.h"
+#include "dbushelper.h"
 #include "kdeconnectconfig.h"
 #include "landevicelink.h"
-#include "lanpairinghandler.h"
-#include "qtcompat_p.h"
-
-#define MIN_VERSION_WITH_SSL_SUPPORT 6
 
 static const int MAX_UNPAIRED_CONNECTIONS = 42;
 static const int MAX_REMEMBERED_IDENTITY_PACKETS = 42;
 
-LanLinkProvider::LanLinkProvider(bool testMode, quint16 udpBroadcastPort, quint16 udpListenPort)
+static const long MILLIS_DELAY_BETWEEN_CONNECTIONS_TO_SAME_DEVICE = 500;
+
+LanLinkProvider::LanLinkProvider(bool testMode)
     : m_server(new Server(this))
     , m_udpSocket(this)
     , m_tcpPort(0)
-    , m_udpBroadcastPort(udpBroadcastPort)
-    , m_udpListenPort(udpListenPort)
     , m_testMode(testMode)
-    , m_combineBroadcastsTimer(this)
+    , m_combineNetworkChangeTimer(this)
+#ifdef KDECONNECT_MDNS
+    , m_mdnsDiscovery(this)
+#endif
 {
-    m_combineBroadcastsTimer.setInterval(0); // increase this if waiting a single event-loop iteration is not enough
-    m_combineBroadcastsTimer.setSingleShot(true);
-    connect(&m_combineBroadcastsTimer, &QTimer::timeout, this, &LanLinkProvider::broadcastToNetwork);
+    m_combineNetworkChangeTimer.setInterval(0); // increase this if waiting a single event-loop iteration is not enough
+    m_combineNetworkChangeTimer.setSingleShot(true);
+    connect(&m_combineNetworkChangeTimer, &QTimer::timeout, this, &LanLinkProvider::combinedOnNetworkChange);
 
     connect(&m_udpSocket, &QIODevice::readyRead, this, &LanLinkProvider::udpBroadcastReceived);
 
@@ -57,17 +63,21 @@ LanLinkProvider::LanLinkProvider(bool testMode, quint16 udpBroadcastPort, quint1
 
     m_udpSocket.setProxy(QNetworkProxy::NoProxy);
 
-    // Detect when a network interface changes status, so we announce ourselves in the new network
-    QNetworkConfigurationManager *networkManager = new QNetworkConfigurationManager(this);
-    connect(networkManager, &QNetworkConfigurationManager::configurationChanged, this, &LanLinkProvider::onNetworkConfigurationChanged);
-}
+    connect(&m_udpSocket, &QAbstractSocket::errorOccurred, [](QAbstractSocket::SocketError socketError) {
+        qWarning() << "Error sending UDP packet:" << socketError;
+    });
 
-void LanLinkProvider::onNetworkConfigurationChanged(const QNetworkConfiguration &config)
-{
-    if (m_lastConfig != config && config.state() == QNetworkConfiguration::Active) {
-        m_lastConfig = config;
-        onNetworkChange();
-    }
+    const auto checkNetworkChange = [this]() {
+        if (QNetworkInformation::instance()->reachability() == QNetworkInformation::Reachability::Online) {
+            onNetworkChange();
+        }
+    };
+    // Detect when a network interface changes status, so we announce ourselves in the new network
+    QNetworkInformation::instance()->loadBackendByFeatures(QNetworkInformation::Feature::Reachability);
+
+    // We want to know if our current network reachability has changed, or if we change from one network to another
+    connect(QNetworkInformation::instance(), &QNetworkInformation::reachabilityChanged, this, checkNetworkChange);
+    connect(QNetworkInformation::instance(), &QNetworkInformation::transportMediumChanged, this, checkNetworkChange);
 }
 
 LanLinkProvider::~LanLinkProvider()
@@ -78,12 +88,12 @@ void LanLinkProvider::onStart()
 {
     const QHostAddress bindAddress = m_testMode ? QHostAddress::LocalHost : QHostAddress::Any;
 
-    bool success = m_udpSocket.bind(bindAddress, m_udpListenPort, QUdpSocket::ShareAddress);
+    bool success = m_udpSocket.bind(bindAddress, UDP_PORT, QUdpSocket::ShareAddress);
     if (!success) {
         QAbstractSocket::SocketError sockErr = m_udpSocket.error();
         // Refer to https://doc.qt.io/qt-5/qabstractsocket.html#SocketError-enum to decode socket error number
         QString errorMessage = QString::fromLatin1(QMetaEnum::fromType<QAbstractSocket::SocketError>().valueToKey(sockErr));
-        qCritical(KDECONNECT_CORE) << QLatin1String("Failed to bind UDP socket on port") << m_udpListenPort << QLatin1String("with error") << errorMessage;
+        qCritical(KDECONNECT_CORE) << "Failed to bind UDP socket on port" << UDP_PORT << "with error" << errorMessage;
     }
     Q_ASSERT(success);
 
@@ -97,12 +107,20 @@ void LanLinkProvider::onStart()
         }
     }
 
-    onNetworkChange();
+    broadcastUdpIdentityPacket();
+
+#ifdef KDECONNECT_MDNS
+    m_mdnsDiscovery.onStart();
+#endif
+
     qCDebug(KDECONNECT_CORE) << "LanLinkProvider started";
 }
 
 void LanLinkProvider::onStop()
 {
+#ifdef KDECONNECT_MDNS
+    m_mdnsDiscovery.onStop();
+#endif
     m_udpSocket.close();
     m_server->close();
     qCDebug(KDECONNECT_CORE) << "LanLinkProvider stopped";
@@ -110,56 +128,38 @@ void LanLinkProvider::onStop()
 
 void LanLinkProvider::onNetworkChange()
 {
-    if (m_combineBroadcastsTimer.isActive()) {
-        qCDebug(KDECONNECT_CORE) << "Preventing duplicate broadcasts";
+    if (m_combineNetworkChangeTimer.isActive()) {
+        qCDebug(KDECONNECT_CORE) << "Device discovery triggered too fast, ignoring";
         return;
     }
-    m_combineBroadcastsTimer.start();
+    m_combineNetworkChangeTimer.start();
 }
 
 // I'm in a new network, let's be polite and introduce myself
-void LanLinkProvider::broadcastToNetwork()
+void LanLinkProvider::combinedOnNetworkChange()
 {
     if (!m_server->isListening()) {
-        // Not started
+        qWarning() << "TCP server not listening, not broadcasting";
         return;
     }
 
     Q_ASSERT(m_tcpPort != 0);
 
-    qCDebug(KDECONNECT_CORE()) << "Broadcasting identity packet";
-
-    QList<QHostAddress> destinations = getBroadcastAddresses();
-
-    NetworkPacket np;
-    NetworkPacket::createIdentityPacket(&np);
-    np.set(QStringLiteral("tcpPort"), m_tcpPort);
-#if defined(Q_OS_MAC) || defined(Q_OS_FREEBSD)
-    // On macOS and FreeBSD, the too large UDP packet (larger than MTU) causes
-    // incomplete transmission.
-    // We remove the capacitilities to reduce the discovery packet to the min
-    // MTU of the interfaces with broadcast feature.
-    int mtu = 1500;
-    for (const QNetworkInterface &iface : QNetworkInterface::allInterfaces()) {
-        if ((iface.flags() & QNetworkInterface::IsUp) && (iface.flags() & QNetworkInterface::IsRunning) && (iface.flags() & QNetworkInterface::CanBroadcast)) {
-            int ifaceMtu = iface.maximumTransmissionUnit();
-            if (ifaceMtu < mtu && ifaceMtu > 0) {
-                mtu = ifaceMtu;
-            }
-        }
-    }
-    QByteArray payload = np.serialize();
-    if (payload.length() > mtu) {
-        // First try to drop the less important outgoing capabilities
-        np.set(QStringLiteral("outgoingCapabilities"), QStringList());
-        payload = np.serialize();
-    }
-    if (payload.length() > mtu) {
-        // If still too large, drop the incoming capabilities
-        np.set(QStringLiteral("incomingCapabilities"), QStringList());
-        payload = np.serialize();
-    }
+    broadcastUdpIdentityPacket();
+#ifdef KDECONNECT_MDNS
+    m_mdnsDiscovery.onNetworkChange();
 #endif
+}
+
+void LanLinkProvider::broadcastUdpIdentityPacket()
+{
+    if (qEnvironmentVariableIsSet("KDECONNECT_DISABLE_UDP_BROADCAST")) {
+        qWarning() << "Not broadcasting UDP because KDECONNECT_DISABLE_UDP_BROADCAST is set";
+        return;
+    }
+    qCDebug(KDECONNECT_CORE) << "Broadcasting identity packet";
+
+    QList<QHostAddress> addresses = getBroadcastAddresses();
 
 #if defined(Q_OS_WIN) || defined(Q_OS_FREEBSD)
     // On Windows and FreeBSD we need to broadcast from every local IP address to reach all networks
@@ -170,16 +170,16 @@ void LanLinkProvider::broadcastToNetwork()
             for (const QNetworkAddressEntry &ifaceAddress : iface.addressEntries()) {
                 QHostAddress sourceAddress = ifaceAddress.ip();
                 if (sourceAddress.protocol() == QAbstractSocket::IPv4Protocol && sourceAddress != QHostAddress::LocalHost) {
-                    qCDebug(KDECONNECT_CORE()) << "Broadcasting as" << sourceAddress;
+                    qCDebug(KDECONNECT_CORE) << "Broadcasting as" << sourceAddress;
                     sendSocket.bind(sourceAddress);
-                    sendBroadcasts(sendSocket, np, destinations);
+                    sendUdpIdentityPacket(sendSocket, addresses);
                     sendSocket.close();
                 }
             }
         }
     }
 #else
-    sendBroadcasts(m_udpSocket, np, destinations);
+    sendUdpIdentityPacket(addresses);
 #endif
 }
 
@@ -206,12 +206,31 @@ QList<QHostAddress> LanLinkProvider::getBroadcastAddresses()
     return destinations;
 }
 
-void LanLinkProvider::sendBroadcasts(QUdpSocket &socket, const NetworkPacket &np, const QList<QHostAddress> &addresses)
+void LanLinkProvider::sendUdpIdentityPacket(const QList<QHostAddress> &addresses)
 {
-    const QByteArray payload = np.serialize();
+    sendUdpIdentityPacket(m_udpSocket, addresses);
+}
+
+void LanLinkProvider::sendUdpIdentityPacket(QUdpSocket &socket, const QList<QHostAddress> &addresses)
+{
+    DeviceInfo myDeviceInfo = KdeConnectConfig::instance().deviceInfo();
+    NetworkPacket identityPacket = myDeviceInfo.toIdentityPacket();
+    identityPacket.set(QStringLiteral("tcpPort"), m_tcpPort);
+    const QByteArray payload = identityPacket.serialize();
 
     for (auto &address : addresses) {
-        socket.writeDatagram(payload, address, m_udpBroadcastPort);
+        qint64 bytes = socket.writeDatagram(payload, address, UDP_PORT);
+        if (bytes == -1 && socket.error() == QAbstractSocket::DatagramTooLargeError) {
+            // On macOS and FreeBSD, UDP broadcasts larger than MTU get dropped. See:
+            // https://opensource.apple.com/source/xnu/xnu-3789.1.32/bsd/netinet/ip_output.c.auto.html#:~:text=/*%20don%27t%20allow%20broadcast%20messages%20to%20be%20fragmented%20*/
+            // We remove the capabilities to reduce the size of the packet.
+            // This should only happen for broadcasts, so UDP packets sent from MDNS discoveries should still work.
+            qWarning() << "Identity packet to" << address << "got rejected because it was too large. Retrying without including the capabilities";
+            identityPacket.set(QStringLiteral("outgoingCapabilities"), QStringList());
+            identityPacket.set(QStringLiteral("incomingCapabilities"), QStringList());
+            const QByteArray smallPayload = identityPacket.serialize();
+            socket.writeDatagram(smallPayload, address, UDP_PORT);
+        }
     }
 }
 
@@ -229,11 +248,9 @@ void LanLinkProvider::udpBroadcastReceived()
         if (sender.isLoopback() && !m_testMode)
             continue;
 
-        NetworkPacket *receivedPacket = new NetworkPacket(QLatin1String(""));
+        NetworkPacket *receivedPacket = new NetworkPacket();
         bool success = NetworkPacket::unserialize(datagram, receivedPacket);
 
-        // qCDebug(KDECONNECT_CORE) << "udp connection from " << receivedPacket->;
-
         // qCDebug(KDECONNECT_CORE) << "Datagram " << datagram.data() ;
 
         if (!success) {
@@ -242,18 +259,28 @@ void LanLinkProvider::udpBroadcastReceived()
             continue;
         }
 
-        if (receivedPacket->type() != PACKET_TYPE_IDENTITY) {
-            qCDebug(KDECONNECT_CORE) << "Received a UDP packet of wrong type" << receivedPacket->type();
+        if (!DeviceInfo::isValidIdentityPacket(receivedPacket)) {
+            qCWarning(KDECONNECT_CORE) << "Invalid identity packet received";
             delete receivedPacket;
             continue;
         }
 
-        if (receivedPacket->get<QString>(QStringLiteral("deviceId")) == KdeConnectConfig::instance().deviceId()) {
+        QString deviceId = receivedPacket->get<QString>(QStringLiteral("deviceId"));
+
+        if (deviceId == KdeConnectConfig::instance().deviceId()) {
             // qCDebug(KDECONNECT_CORE) << "Ignoring my own broadcast";
             delete receivedPacket;
             continue;
         }
 
+        qint64 now = QDateTime::currentMSecsSinceEpoch();
+        if (m_lastConnectionTime[deviceId] + MILLIS_DELAY_BETWEEN_CONNECTIONS_TO_SAME_DEVICE > now) {
+            qCDebug(KDECONNECT_CORE) << "Discarding second UPD packet from the same device" << deviceId << "received too quickly";
+            delete receivedPacket;
+            return;
+        }
+        m_lastConnectionTime[deviceId] = now;
+
         int tcpPort = receivedPacket->get<int>(QStringLiteral("tcpPort"));
         if (tcpPort < MIN_TCP_PORT || tcpPort > MAX_TCP_PORT) {
             qCDebug(KDECONNECT_CORE) << "TCP port outside of kdeconnect's range";
@@ -275,11 +302,10 @@ void LanLinkProvider::udpBroadcastReceived()
         m_receivedIdentityPackets[socket].np = receivedPacket;
         m_receivedIdentityPackets[socket].sender = sender;
         connect(socket, &QAbstractSocket::connected, this, &LanLinkProvider::tcpSocketConnected);
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-        connect(socket, QOverload<QAbstractSocket::SocketError>::of(&QAbstractSocket::error), this, &LanLinkProvider::connectError);
-#else
         connect(socket, &QAbstractSocket::errorOccurred, this, &LanLinkProvider::connectError);
-#endif
+        connect(socket, &QObject::destroyed, this, [this, socket]() {
+            delete m_receivedIdentityPackets.take(socket).np;
+        });
         socket->connectToHost(sender, tcpPort);
     }
 }
@@ -292,14 +318,12 @@ void LanLinkProvider::connectError(QAbstractSocket::SocketError socketError)
 
     qCDebug(KDECONNECT_CORE) << "Socket error" << socketError;
     qCDebug(KDECONNECT_CORE) << "Fallback (1), try reverse connection (send udp packet)" << socket->errorString();
-    NetworkPacket np(QLatin1String(""));
-    NetworkPacket::createIdentityPacket(&np);
+    NetworkPacket np = KdeConnectConfig::instance().deviceInfo().toIdentityPacket();
     np.set(QStringLiteral("tcpPort"), m_tcpPort);
-    m_udpSocket.writeDatagram(np.serialize(), m_receivedIdentityPackets[socket].sender, m_udpBroadcastPort);
+    m_udpSocket.writeDatagram(np.serialize(), m_receivedIdentityPackets[socket].sender, UDP_PORT);
 
     // The socket we created didn't work, and we didn't manage
     // to create a LanDeviceLink from it, deleting everything.
-    delete m_receivedIdentityPackets.take(socket).np;
     socket->deleteLater();
 }
 
@@ -308,14 +332,11 @@ void LanLinkProvider::tcpSocketConnected()
 {
     QSslSocket *socket = qobject_cast<QSslSocket *>(sender());
 
-    if (!socket)
+    if (!socket) {
         return;
-        // TODO Delete me?
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-    disconnect(socket, QOverload<QAbstractSocket::SocketError>::of(&QAbstractSocket::error), this, &LanLinkProvider::connectError);
-#else
+    }
+
     disconnect(socket, &QAbstractSocket::errorOccurred, this, &LanLinkProvider::connectError);
-#endif
 
     configureSocket(socket);
 
@@ -327,8 +348,7 @@ void LanLinkProvider::tcpSocketConnected()
     // qCDebug(KDECONNECT_CORE) << "tcpSocketConnected" << socket->isWritable();
 
     // If network is on ssl, do not believe when they are connected, believe when handshake is completed
-    NetworkPacket np2(QLatin1String(""));
-    NetworkPacket::createIdentityPacket(&np2);
+    NetworkPacket np2 = KdeConnectConfig::instance().deviceInfo().toIdentityPacket();
     socket->write(np2.serialize());
     bool success = socket->waitForBytesWritten();
 
@@ -336,33 +356,25 @@ void LanLinkProvider::tcpSocketConnected()
         qCDebug(KDECONNECT_CORE) << "TCP connection done (i'm the existing device)";
 
         // if ssl supported
-        if (receivedPacket->get<int>(QStringLiteral("protocolVersion")) >= MIN_VERSION_WITH_SSL_SUPPORT) {
-            bool isDeviceTrusted = KdeConnectConfig::instance().trustedDevices().contains(deviceId);
-            configureSslSocket(socket, deviceId, isDeviceTrusted);
+        bool isDeviceTrusted = KdeConnectConfig::instance().trustedDevices().contains(deviceId);
+        configureSslSocket(socket, deviceId, isDeviceTrusted);
 
-            qCDebug(KDECONNECT_CORE) << "Starting server ssl (I'm the client TCP socket)";
+        qCDebug(KDECONNECT_CORE) << "Starting server ssl (I'm the client TCP socket)";
 
-            connect(socket, &QSslSocket::encrypted, this, &LanLinkProvider::encrypted);
+        connect(socket, &QSslSocket::encrypted, this, &LanLinkProvider::encrypted);
 
-            connect(socket, QOverload<const QList<QSslError> &>::of(&QSslSocket::sslErrors), this, &LanLinkProvider::sslErrors);
-
-            socket->startServerEncryption();
-
-            return; // Return statement prevents from deleting received packet, needed in slot "encrypted"
-        } else {
-            qWarning() << receivedPacket->get<QString>(QStringLiteral("deviceName")) << "uses an old protocol version, this won't work";
-            // addLink(deviceId, socket, receivedPacket, LanDeviceLink::Remotely);
-        }
+        connect(socket, &QSslSocket::sslErrors, this, &LanLinkProvider::sslErrors);
 
+        socket->startServerEncryption();
     } else {
-        // I think this will never happen, but if it happens the deviceLink
-        //(or the socket that is now inside it) might not be valid. Delete them.
-        qCDebug(KDECONNECT_CORE) << "Fallback (2), try reverse connection (send udp packet)";
-        m_udpSocket.writeDatagram(np2.serialize(), m_receivedIdentityPackets[socket].sender, m_udpBroadcastPort);
-    }
+        // The socket doesn't seem to work, so we can't create the connection.
 
-    delete m_receivedIdentityPackets.take(socket).np;
-    // We don't delete the socket because now it's owned by the LanDeviceLink
+        qCDebug(KDECONNECT_CORE) << "Fallback (2), try reverse connection (send udp packet)";
+        m_udpSocket.writeDatagram(np2.serialize(), m_receivedIdentityPackets[socket].sender, UDP_PORT);
+
+        // Disconnect should trigger deleteLater, which should remove the socket from m_receivedIdentityPackets
+        socket->disconnectFromHost();
+    }
 }
 
 void LanLinkProvider::encrypted()
@@ -374,21 +386,16 @@ void LanLinkProvider::encrypted()
         return;
 
     Q_ASSERT(socket->mode() != QSslSocket::UnencryptedMode);
-    LanDeviceLink::ConnectionStarted connectionOrigin = (socket->mode() == QSslSocket::SslClientMode) ? LanDeviceLink::Locally : LanDeviceLink::Remotely;
 
-    NetworkPacket *receivedPacket = m_receivedIdentityPackets[socket].np;
-    const QString &deviceId = socket->peerCertificate().subjectDisplayName();
+    NetworkPacket *identityPacket = m_receivedIdentityPackets[socket].np;
 
-    if (m_links.contains(deviceId) && m_links[deviceId]->certificate() != socket->peerCertificate()) {
-        socket->disconnectFromHost();
-        qCWarning(KDECONNECT_CORE) << "Got connection for the same deviceId but certificates don't match. Ignoring " << deviceId;
-        return;
-    }
+    DeviceInfo deviceInfo = DeviceInfo::FromIdentityPacketAndCert(*identityPacket, socket->peerCertificate());
 
-    addLink(deviceId, socket, receivedPacket, connectionOrigin);
-
-    // Copied from tcpSocketConnected slot, now delete received packet
+    // We don't delete the socket because now it's owned by the LanDeviceLink
+    disconnect(socket, &QObject::destroyed, nullptr, nullptr);
     delete m_receivedIdentityPackets.take(socket).np;
+
+    addLink(socket, deviceInfo);
 }
 
 void LanLinkProvider::sslErrors(const QList<QSslError> &errors)
@@ -408,8 +415,8 @@ void LanLinkProvider::sslErrors(const QList<QSslError> &errors)
     }
 
     if (fatal) {
+        // Disconnect should trigger deleteLater, which should remove the socket from m_receivedIdentityPackets
         socket->disconnectFromHost();
-        delete m_receivedIdentityPackets.take(socket).np;
     }
 }
 
@@ -440,7 +447,7 @@ void LanLinkProvider::newConnection()
     }
 }
 
-// I'm the new device and this is the answer to my UDP identity packet (data received)
+// I'm the new device and this is the TCP response to my UDP identity packet
 void LanLinkProvider::dataReceived()
 {
     QSslSocket *socket = qobject_cast<QSslSocket *>(sender());
@@ -453,36 +460,25 @@ void LanLinkProvider::dataReceived()
         return;
     }
 
-#if QT_VERSION < QT_VERSION_CHECK(5, 7, 0)
-    if (!socket->canReadLine())
+    if (!socket->canReadLine()) {
+        // This can happen if the packet is large enough to be split in two chunks
         return;
-#else
-    socket->startTransaction();
-#endif
+    }
 
     const QByteArray data = socket->readLine();
 
     qCDebug(KDECONNECT_CORE) << "LanLinkProvider received reply:" << data;
 
-    NetworkPacket *np = new NetworkPacket(QLatin1String(""));
+    NetworkPacket *np = new NetworkPacket();
     bool success = NetworkPacket::unserialize(data, np);
 
-#if QT_VERSION < QT_VERSION_CHECK(5, 7, 0)
     if (!success) {
         delete np;
         return;
     }
-#else
-    if (!success) {
-        delete np;
-        socket->rollbackTransaction();
-        return;
-    }
-    socket->commitTransaction();
-#endif
 
-    if (np->type() != PACKET_TYPE_IDENTITY) {
-        qCWarning(KDECONNECT_CORE) << "LanLinkProvider/newConnection: Expected identity, received " << np->type();
+    if (!DeviceInfo::isValidIdentityPacket(np)) {
+        qCWarning(KDECONNECT_CORE) << "Invalid identity packet received";
         delete np;
         return;
     }
@@ -495,6 +491,9 @@ void LanLinkProvider::dataReceived()
 
     // Needed in "encrypted" if ssl is used, similar to "tcpSocketConnected"
     m_receivedIdentityPackets[socket].np = np;
+    connect(socket, &QObject::destroyed, this, [this, socket]() {
+        delete m_receivedIdentityPackets.take(socket).np;
+    });
 
     const QString &deviceId = np->get<QString>(QStringLiteral("deviceId"));
     // qCDebug(KDECONNECT_CORE) << "Handshaking done (i'm the new device)";
@@ -502,67 +501,37 @@ void LanLinkProvider::dataReceived()
     // This socket will now be owned by the LanDeviceLink or we don't want more data to be received, forget about it
     disconnect(socket, &QIODevice::readyRead, this, &LanLinkProvider::dataReceived);
 
-    if (np->get<int>(QStringLiteral("protocolVersion")) >= MIN_VERSION_WITH_SSL_SUPPORT) {
-        bool isDeviceTrusted = KdeConnectConfig::instance().trustedDevices().contains(deviceId);
-        configureSslSocket(socket, deviceId, isDeviceTrusted);
+    bool isDeviceTrusted = KdeConnectConfig::instance().trustedDevices().contains(deviceId);
+    configureSslSocket(socket, deviceId, isDeviceTrusted);
 
-        qCDebug(KDECONNECT_CORE) << "Starting client ssl (but I'm the server TCP socket)";
+    qCDebug(KDECONNECT_CORE) << "Starting client ssl (but I'm the server TCP socket)";
 
-        connect(socket, &QSslSocket::encrypted, this, &LanLinkProvider::encrypted);
+    connect(socket, &QSslSocket::encrypted, this, &LanLinkProvider::encrypted);
 
-        if (isDeviceTrusted) {
-            connect(socket, QOverload<const QList<QSslError> &>::of(&QSslSocket::sslErrors), this, &LanLinkProvider::sslErrors);
-        }
-
-        socket->startClientEncryption();
-
-    } else {
-        qWarning() << np->get<QString>(QStringLiteral("deviceName")) << "uses an old protocol version, this won't work";
-        // addLink(deviceId, socket, np, LanDeviceLink::Locally);
-        delete m_receivedIdentityPackets.take(socket).np;
+    if (isDeviceTrusted) {
+        connect(socket, &QSslSocket::sslErrors, this, &LanLinkProvider::sslErrors);
     }
+
+    socket->startClientEncryption();
 }
 
-void LanLinkProvider::deviceLinkDestroyed(QObject *destroyedDeviceLink)
+void LanLinkProvider::onLinkDestroyed(const QString &deviceId, DeviceLink *oldPtr)
 {
-    const QString id = destroyedDeviceLink->property("deviceId").toString();
-    // qCDebug(KDECONNECT_CORE) << "deviceLinkDestroyed" << id;
-    QMap<QString, LanDeviceLink *>::iterator linkIterator = m_links.find(id);
-    Q_ASSERT(linkIterator != m_links.end());
-    if (linkIterator != m_links.end()) {
-        Q_ASSERT(linkIterator.value() == destroyedDeviceLink);
-        m_links.erase(linkIterator);
-        auto pairingHandler = m_pairingHandlers.take(id);
-        if (pairingHandler) {
-            pairingHandler->deleteLater();
-        }
-    }
+    qCDebug(KDECONNECT_CORE) << "LanLinkProvider deviceLinkDestroyed" << deviceId;
+    DeviceLink *link = m_links.take(deviceId);
+    Q_ASSERT(link == oldPtr);
 }
 
 void LanLinkProvider::configureSslSocket(QSslSocket *socket, const QString &deviceId, bool isDeviceTrusted)
 {
-    // Setting supported ciphers manually, to match those on Android (FIXME: Test if this can be left unconfigured and still works for Android 4)
-    QList<QSslCipher> socketCiphers;
-    socketCiphers.append(QSslCipher(QStringLiteral("ECDHE-ECDSA-AES256-GCM-SHA384")));
-    socketCiphers.append(QSslCipher(QStringLiteral("ECDHE-ECDSA-AES128-GCM-SHA256")));
-    socketCiphers.append(QSslCipher(QStringLiteral("ECDHE-RSA-AES128-SHA")));
-
     // Configure for ssl
     QSslConfiguration sslConfig;
-    sslConfig.setCiphers(socketCiphers);
     sslConfig.setLocalCertificate(KdeConnectConfig::instance().certificate());
-
-    QFile privateKeyFile(KdeConnectConfig::instance().privateKeyPath());
-    QSslKey privateKey;
-    if (privateKeyFile.open(QIODevice::ReadOnly)) {
-        privateKey = QSslKey(privateKeyFile.readAll(), QSsl::Rsa);
-    }
-    privateKeyFile.close();
-    sslConfig.setPrivateKey(privateKey);
+    sslConfig.setPrivateKey(KdeConnectConfig::instance().privateKey());
 
     if (isDeviceTrusted) {
-        QString certString = KdeConnectConfig::instance().getDeviceProperty(deviceId, QStringLiteral("certificate"), QString());
-        sslConfig.setCaCertificates({QSslCertificate(certString.toLatin1())});
+        QSslCertificate certificate = KdeConnectConfig::instance().getTrustedDeviceCertificate(deviceId);
+        sslConfig.setCaCertificates({certificate});
         sslConfig.setPeerVerifyMode(QSslSocket::VerifyPeer);
     } else {
         sslConfig.setPeerVerifyMode(QSslSocket::QueryPeer);
@@ -571,100 +540,58 @@ void LanLinkProvider::configureSslSocket(QSslSocket *socket, const QString &devi
     socket->setPeerVerifyName(deviceId);
 
     // Usually SSL errors are only bad for trusted devices. Uncomment this section to log errors in any case, for debugging.
-    // QObject::connect(socket, static_cast<void (QSslSocket::*)(const QList<QSslError>&)>(&QSslSocket::sslErrors), [](const QList<QSslError>& errors)
-    //{
-    //     Q_FOREACH (const QSslError& error, errors) {
-    //         qCDebug(KDECONNECT_CORE) << "SSL Error:" << error.errorString();
-    //     }
+    // connect(socket, &QSslSocket::sslErrors, [](const QList<QSslError>& errors)
+    // {
+    //      for (const QSslError& error : errors) {
+    //          qCDebug(KDECONNECT_CORE) << "SSL Error:" << error.errorString();
+    //      }
     // });
 }
 
 void LanLinkProvider::configureSocket(QSslSocket *socket)
 {
     socket->setProxy(QNetworkProxy::NoProxy);
-
     socket->setSocketOption(QAbstractSocket::KeepAliveOption, QVariant(1));
-
-#ifdef TCP_KEEPIDLE
-    // time to start sending keepalive packets (seconds)
-    int maxIdle = 10;
-    setsockopt(socket->socketDescriptor(), IPPROTO_TCP, TCP_KEEPIDLE, &maxIdle, sizeof(maxIdle));
-#endif
-
-#ifdef TCP_KEEPINTVL
-    // interval between keepalive packets after the initial period (seconds)
-    int interval = 5;
-    setsockopt(socket->socketDescriptor(), IPPROTO_TCP, TCP_KEEPINTVL, &interval, sizeof(interval));
-#endif
-
-#ifdef TCP_KEEPCNT
-    // number of missed keepalive packets before disconnecting
-    int count = 3;
-    setsockopt(socket->socketDescriptor(), IPPROTO_TCP, TCP_KEEPCNT, &count, sizeof(count));
-#endif
 }
 
-void LanLinkProvider::addLink(const QString &deviceId, QSslSocket *socket, NetworkPacket *receivedPacket, LanDeviceLink::ConnectionStarted connectionOrigin)
+void LanLinkProvider::addLink(QSslSocket *socket, const DeviceInfo &deviceInfo)
 {
+    QString certDeviceId = socket->peerCertificate().subjectDisplayName();
+    DBusHelper::filterNonExportableCharacters(certDeviceId);
+    if (deviceInfo.id != certDeviceId) {
+        socket->disconnectFromHost();
+        qCWarning(KDECONNECT_CORE) << "DeviceID in cert doesn't match deviceID in identity packet. " << deviceInfo.id << " vs " << certDeviceId;
+        return;
+    }
+
     // Socket disconnection will now be handled by LanDeviceLink
     disconnect(socket, &QAbstractSocket::disconnected, socket, &QObject::deleteLater);
 
     LanDeviceLink *deviceLink;
     // Do we have a link for this device already?
-    QMap<QString, LanDeviceLink *>::iterator linkIterator = m_links.find(deviceId);
+    QMap<QString, LanDeviceLink *>::iterator linkIterator = m_links.find(deviceInfo.id);
     if (linkIterator != m_links.end()) {
-        // qCDebug(KDECONNECT_CORE) << "Reusing link to" << deviceId;
         deviceLink = linkIterator.value();
-        deviceLink->reset(socket, connectionOrigin);
+        if (deviceLink->deviceInfo().certificate != deviceInfo.certificate) {
+            qWarning() << "LanLink was asked to replace a socket but the certificate doesn't match, aborting";
+            return;
+        }
+        // qCDebug(KDECONNECT_CORE) << "Reusing link to" << deviceId;
+        deviceLink->reset(socket);
     } else {
-        deviceLink = new LanDeviceLink(deviceId, this, socket, connectionOrigin);
+        deviceLink = new LanDeviceLink(deviceInfo, this, socket);
         // Socket disconnection will now be handled by LanDeviceLink
         disconnect(socket, &QAbstractSocket::disconnected, socket, &QObject::deleteLater);
-        bool isDeviceTrusted = KdeConnectConfig::instance().trustedDevices().contains(deviceId);
+        bool isDeviceTrusted = KdeConnectConfig::instance().trustedDevices().contains(deviceInfo.id);
         if (!isDeviceTrusted && m_links.size() > MAX_UNPAIRED_CONNECTIONS) {
-            qCWarning(KDECONNECT_CORE) << "Too many unpaired devices to remember them all. Ignoring " << deviceId;
+            qCWarning(KDECONNECT_CORE) << "Too many unpaired devices to remember them all. Ignoring " << deviceInfo.id;
             socket->disconnectFromHost();
             socket->deleteLater();
             return;
         }
-        connect(deviceLink, &QObject::destroyed, this, &LanLinkProvider::deviceLinkDestroyed);
-        m_links[deviceId] = deviceLink;
-        if (m_pairingHandlers.contains(deviceId)) {
-            // We shouldn't have a pairinghandler if we didn't have a link.
-            // Crash if debug, recover if release (by setting the new devicelink to the old pairinghandler)
-            Q_ASSERT(m_pairingHandlers.contains(deviceId));
-            m_pairingHandlers[deviceId]->setDeviceLink(deviceLink);
-        }
+        m_links[deviceInfo.id] = deviceLink;
     }
-    Q_EMIT onConnectionReceived(*receivedPacket, deviceLink);
+    Q_EMIT onConnectionReceived(deviceLink);
 }
 
-LanPairingHandler *LanLinkProvider::createPairingHandler(DeviceLink *link)
-{
-    LanPairingHandler *ph = m_pairingHandlers.value(link->deviceId());
-    if (!ph) {
-        ph = new LanPairingHandler(link);
-        qCDebug(KDECONNECT_CORE) << "creating pairing handler for" << link->deviceId();
-        connect(ph, &LanPairingHandler::pairingError, link, &DeviceLink::pairingError);
-        m_pairingHandlers[link->deviceId()] = ph;
-    }
-    return ph;
-}
-
-void LanLinkProvider::userRequestsPair(const QString &deviceId)
-{
-    LanPairingHandler *ph = createPairingHandler(m_links.value(deviceId));
-    ph->requestPairing();
-}
-
-void LanLinkProvider::userRequestsUnpair(const QString &deviceId)
-{
-    LanPairingHandler *ph = createPairingHandler(m_links.value(deviceId));
-    ph->unpair();
-}
-
-void LanLinkProvider::incomingPairPacket(DeviceLink *deviceLink, const NetworkPacket &np)
-{
-    LanPairingHandler *ph = createPairingHandler(deviceLink);
-    ph->packetReceived(np);
-}
+#include "moc_lanlinkprovider.cpp"
