@@ -7,7 +7,7 @@
 #ifndef LANLINKPROVIDER_H
 #define LANLINKPROVIDER_H
 
-#include <QNetworkSession>
+#include <QNetworkInformation>
 #include <QObject>
 #include <QSslSocket>
 #include <QTcpServer>
@@ -18,8 +18,10 @@
 #include "kdeconnectcore_export.h"
 #include "landevicelink.h"
 #include "server.h"
+#ifdef KDECONNECT_MDNS
+#include "mdnsdiscovery.h"
+#endif
 
-class LanPairingHandler;
 class KDECONNECTCORE_EXPORT LanLinkProvider : public LinkProvider
 {
     Q_OBJECT
@@ -27,24 +29,21 @@ class KDECONNECTCORE_EXPORT LanLinkProvider : public LinkProvider
 public:
     /**
      * @param testMode Some special overrides needed while testing
-     * @param udpBroadcastPort Port which should be used for *sending* identity packets
-     * @param udpListenPort Port which should be used for *receiving* identity packets
      */
-    LanLinkProvider(bool testMode = false, quint16 udpBroadcastPort = UDP_PORT, quint16 udpListenPort = UDP_PORT);
+    LanLinkProvider(bool testMode = false);
     ~LanLinkProvider() override;
 
     QString name() override
     {
         return QStringLiteral("LanLinkProvider");
     }
+
     int priority() override
     {
-        return PRIORITY_HIGH;
+        return 20;
     }
 
-    void userRequestsPair(const QString &deviceId);
-    void userRequestsUnpair(const QString &deviceId);
-    void incomingPairPacket(DeviceLink *device, const NetworkPacket &np);
+    void sendUdpIdentityPacket(const QList<QHostAddress> &addresses);
 
     static void configureSslSocket(QSslSocket *socket, const QString &deviceId, bool isDeviceTrusted);
     static void configureSocket(QSslSocket *socket);
@@ -58,6 +57,7 @@ public:
 
 public Q_SLOTS:
     void onNetworkChange() override;
+    void onLinkDestroyed(const QString &deviceId, DeviceLink *oldPtr) override;
     void onStart() override;
     void onStop() override;
     void tcpSocketConnected();
@@ -68,36 +68,33 @@ private Q_SLOTS:
     void udpBroadcastReceived();
     void newConnection();
     void dataReceived();
-    void deviceLinkDestroyed(QObject *destroyedDeviceLink);
     void sslErrors(const QList<QSslError> &errors);
-    void broadcastToNetwork();
+    void combinedOnNetworkChange();
 
 private:
-    LanPairingHandler *createPairingHandler(DeviceLink *link);
-
-    void onNetworkConfigurationChanged(const QNetworkConfiguration &config);
-    void addLink(const QString &deviceId, QSslSocket *socket, NetworkPacket *receivedPacket, LanDeviceLink::ConnectionStarted connectionOrigin);
+    void addLink(QSslSocket *socket, const DeviceInfo &deviceInfo);
     QList<QHostAddress> getBroadcastAddresses();
-    void sendBroadcasts(QUdpSocket &socket, const NetworkPacket &np, const QList<QHostAddress> &addresses);
+    void sendUdpIdentityPacket(QUdpSocket &socket, const QList<QHostAddress> &addresses);
+    void broadcastUdpIdentityPacket();
 
     Server *m_server;
     QUdpSocket m_udpSocket;
     quint16 m_tcpPort;
 
-    quint16 m_udpBroadcastPort;
-    quint16 m_udpListenPort;
-
     QMap<QString, LanDeviceLink *> m_links;
-    QMap<QString, LanPairingHandler *> m_pairingHandlers;
 
     struct PendingConnect {
         NetworkPacket *np;
         QHostAddress sender;
     };
     QMap<QSslSocket *, PendingConnect> m_receivedIdentityPackets;
-    QNetworkConfiguration m_lastConfig;
+    QMap<QString, qint64> m_lastConnectionTime;
     const bool m_testMode;
-    QTimer m_combineBroadcastsTimer;
+    QTimer m_combineNetworkChangeTimer;
+
+#ifdef KDECONNECT_MDNS
+    MdnsDiscovery m_mdnsDiscovery;
+#endif
 };
 
 #endif
