@@ -24,16 +24,18 @@ public:
     }
     int priority() override
     {
-        return PRIORITY_LOW;
+        return 0;
     }
 
     void onStart() override;
     void onStop() override;
     void onNetworkChange() override;
+    void onLinkDestroyed(const QString &, DeviceLink *) override
+    {
+    }
 
 private:
     QPointer<LoopbackDeviceLink> loopbackDeviceLink;
-    NetworkPacket identityPacket;
 };
 
 #endif
