@@ -28,9 +28,6 @@
 #include "backends/loopback/loopbacklinkprovider.h"
 #include "device.h"
 
-// In older Qt released, qAsConst isnt available
-#include "qtcompat_p.h"
-
 static Daemon *s_instance = nullptr;
 
 struct DaemonPrivate {
@@ -65,6 +62,16 @@ void Daemon::init()
 {
     qCDebug(KDECONNECT_CORE) << "Daemon starting";
 
+    // Register on DBus
+    // This must happen as early as possible in the process startup to ensure
+    // the absolute minimum amount of blocking on logon/autostart
+
+    qDBusRegisterMetaType<QMap<QString, QString>>();
+    QDBusConnection::sessionBus().registerService(QStringLiteral("org.kde.kdeconnect"));
+    QDBusConnection::sessionBus().registerObject(QStringLiteral("/modules/kdeconnect"), this, QDBusConnection::ExportScriptableContents);
+
+    qCDebug(KDECONNECT_CORE) << "DBus registration complete";
+
     // Load backends
     if (d->m_testMode)
         d->m_linkProviders.insert(new LoopbackLinkProvider());
@@ -78,23 +85,31 @@ void Daemon::init()
 #endif
     }
 
-    // Register on DBus
-    qDBusRegisterMetaType<QMap<QString, QString>>();
-    QDBusConnection::sessionBus().registerService(QStringLiteral("org.kde.kdeconnect"));
-    QDBusConnection::sessionBus().registerObject(QStringLiteral("/modules/kdeconnect"), this, QDBusConnection::ExportScriptableContents);
+    qCDebug(KDECONNECT_CORE) << "Backends loaded";
 
     // Read remembered paired devices
     const QStringList &list = KdeConnectConfig::instance().trustedDevices();
     for (const QString &id : list) {
-        addDevice(new Device(this, id));
+        Device *d = new Device(this, id);
+        // prune away devices with malformed certificates
+        if (d->hasInvalidCertificate()) {
+            qCDebug(KDECONNECT_CORE) << "Certificate for device " << id << "illegal, deleting the device";
+            KdeConnectConfig::instance().removeTrustedDevice(id);
+        } else {
+            addDevice(d);
+        }
     }
 
+    qCDebug(KDECONNECT_CORE) << "Paired devices added";
+
     // Listen to new devices
-    for (LinkProvider *a : qAsConst(d->m_linkProviders)) {
+    for (LinkProvider *a : std::as_const(d->m_linkProviders)) {
         connect(a, &LinkProvider::onConnectionReceived, this, &Daemon::onNewDeviceLink);
         a->onStart();
     }
 
+    qCDebug(KDECONNECT_CORE) << "Link providers started";
+
     NotificationServerInfo::instance().init();
 
     qCDebug(KDECONNECT_CORE) << "Daemon started";
@@ -111,14 +126,15 @@ void Daemon::removeDevice(Device *device)
 void Daemon::forceOnNetworkChange()
 {
     qCDebug(KDECONNECT_CORE) << "Sending onNetworkChange to" << d->m_linkProviders.size() << "LinkProviders";
-    for (LinkProvider *a : qAsConst(d->m_linkProviders)) {
+    for (LinkProvider *a : std::as_const(d->m_linkProviders)) {
+        qCDebug(KDECONNECT_CORE) << "Sending onNetworkChange to: " << a->name();
         a->onNetworkChange();
     }
 }
 
 Device *Daemon::getDevice(const QString &deviceId)
 {
-    for (Device *device : qAsConst(d->m_devices)) {
+    for (Device *device : std::as_const(d->m_devices)) {
         if (device->id() == deviceId) {
             return device;
         }
@@ -126,7 +142,7 @@ Device *Daemon::getDevice(const QString &deviceId)
     return nullptr;
 }
 
-const QSet<LinkProvider *> &Daemon::getLinkProviders() const
+QSet<LinkProvider *> Daemon::getLinkProviders() const
 {
     return d->m_linkProviders;
 }
@@ -134,7 +150,7 @@ const QSet<LinkProvider *> &Daemon::getLinkProviders() const
 QStringList Daemon::devices(bool onlyReachable, bool onlyTrusted) const
 {
     QStringList ret;
-    for (Device *device : qAsConst(d->m_devices)) {
+    for (Device *device : std::as_const(d->m_devices)) {
         if (onlyReachable && !device->isReachable())
             continue;
         if (onlyTrusted && !device->isPaired())
@@ -147,7 +163,7 @@ QStringList Daemon::devices(bool onlyReachable, bool onlyTrusted) const
 QMap<QString, QString> Daemon::deviceNames(bool onlyReachable, bool onlyTrusted) const
 {
     QMap<QString, QString> ret;
-    for (Device *device : qAsConst(d->m_devices)) {
+    for (Device *device : std::as_const(d->m_devices)) {
         if (onlyReachable && !device->isReachable())
             continue;
         if (onlyTrusted && !device->isPaired())
@@ -159,9 +175,9 @@ QMap<QString, QString> Daemon::deviceNames(bool onlyReachable, bool onlyTrusted)
 
 void Daemon::onNewDeviceLink(DeviceLink *link)
 {
-    const QString &id = link->deviceId();
+    QString id = link->deviceId();
 
-    // qCDebug(KDECONNECT_CORE) << "Device discovered" << id << "via" << dl->name();
+    qCDebug(KDECONNECT_CORE) << "Device discovered" << id << "via link with priority" << link->priority();
 
     if (d->m_devices.contains(id)) {
         qCDebug(KDECONNECT_CORE) << "It is a known device" << link->deviceInfo().name;
@@ -196,10 +212,11 @@ void Daemon::onDeviceStatusChanged()
 
 void Daemon::setAnnouncedName(const QString &name)
 {
+    QString filteredName = DeviceInfo::filterName(name);
     qCDebug(KDECONNECT_CORE) << "Announcing name";
-    KdeConnectConfig::instance().setName(name);
+    KdeConnectConfig::instance().setName(filteredName);
     forceOnNetworkChange();
-    Q_EMIT announcedNameChanged(name);
+    Q_EMIT announcedNameChanged(filteredName);
 }
 
 void Daemon::setCustomDevices(const QStringList &addresses)
@@ -242,7 +259,7 @@ QList<Device *> Daemon::devicesList() const
 
 QString Daemon::deviceIdByName(const QString &name) const
 {
-    for (Device *device : qAsConst(d->m_devices)) {
+    for (Device *device : std::as_const(d->m_devices)) {
         if (device->name() == name && device->isPaired())
             return device->id();
     }
@@ -269,7 +286,7 @@ void Daemon::addDevice(Device *device)
 QStringList Daemon::pairingRequests() const
 {
     QStringList ret;
-    for (Device *dev : qAsConst(d->m_devices)) {
+    for (Device *dev : std::as_const(d->m_devices)) {
         if (dev->isPairRequestedByPeer())
             ret += dev->id();
     }
@@ -284,3 +301,5 @@ QString Daemon::selfId() const
 {
     return KdeConnectConfig::instance().deviceId();
 }
+
+#include "moc_daemon.cpp"
