@@ -8,6 +8,7 @@
 #define KDECONNECTCONFIG_H
 
 #include <QDir>
+#include <QSslKey>
 
 #include "deviceinfo.h"
 #include "kdeconnectcore_export.h"
@@ -26,8 +27,10 @@ public:
     QString deviceId();
     QString name();
     DeviceType deviceType();
+    QSslKey privateKey();
     QSslCertificate certificate();
     DeviceInfo deviceInfo();
+    QSsl::KeyAlgorithm privateKeyAlgorithm();
     QString privateKeyPath();
     QString certificatePath();
 
@@ -58,13 +61,6 @@ public:
     QDir deviceConfigDir(const QString &deviceId);
     QDir pluginConfigDir(const QString &deviceId, const QString &pluginName); // Used by KdeConnectPluginConfig
 
-#ifdef Q_OS_MAC
-    /*
-     * Get private DBus Address when use private DBus
-     */
-    QString privateDBusAddressPath();
-    QString privateDBusAddress();
-#endif
 private:
     KdeConnectConfig();
 
@@ -74,6 +70,8 @@ private:
     void generatePrivateKey(const QString &path);
     void generateCertificate(const QString &path);
 
+    void removeAllTrustedDevices();
+
     struct KdeConnectConfigPrivate *d;
 };
 
