@@ -4,12 +4,13 @@
  * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef KDECONNECTKCM_H
-#define KDECONNECTKCM_H
+#pragma once
 
 #include <KCModule>
 #include <QStandardItemModel>
+#include <kconfigwidgets_version.h>
 
+#include "ui_kcm.h"
 #include <core/pairstate.h>
 
 class QModelIndex;
@@ -18,29 +19,21 @@ class DaemonDbusInterface;
 class DevicesModel;
 class DevicesSortProxyModel;
 
-namespace Ui
-{
-class KdeConnectKcmUi;
-}
-
 class KdeConnectKcm : public KCModule
 {
     Q_OBJECT
 public:
-    KdeConnectKcm(QWidget *parent, const QVariantList &);
+    KdeConnectKcm(QObject *parent, const KPluginMetaData &md, const QVariantList &args);
     ~KdeConnectKcm() override;
 
 private:
     void save() override;
-    QSize sizeHint() const override;
-    QSize minimumSizeHint() const override;
 
 private Q_SLOTS:
-    void deviceSelected(const QModelIndex &current);
+    void deviceSelected(const QString &deviceId);
     void requestPairing();
     void pluginsConfigChanged(bool changed);
     void sendPing();
-    void resetSelection();
     void pairingFailed(const QString &error);
     void refresh();
     void renameShow();
@@ -54,16 +47,13 @@ private Q_SLOTS:
 private:
     void resetDeviceView();
 
-    Ui::KdeConnectKcmUi *kcmUi;
+    Ui::KdeConnectKcmUi kcmUi;
     DaemonDbusInterface *daemon;
     DevicesModel *devicesModel;
     DevicesSortProxyModel *sortProxyModel;
     DeviceDbusInterface *currentDevice;
-    QModelIndex currentIndex;
     QStringList m_oldSupportedPluginNames;
 
 public Q_SLOTS:
     void unpair();
 };
-
-#endif
