@@ -5,30 +5,22 @@
  * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef BIGSCREENPLUGIN_H
-#define BIGSCREENPLUGIN_H
+#pragma once
 
 #include <core/kdeconnectplugin.h>
 
 #define PACKET_TYPE_BIGSCREEN_STT QStringLiteral("kdeconnect.bigscreen.stt")
 
-class Q_DECL_EXPORT BigscreenPlugin
-    : public KdeConnectPlugin
+class BigscreenPlugin : public KdeConnectPlugin
 {
     Q_OBJECT
     Q_CLASSINFO("D-Bus Interface", "org.kde.kdeconnect.device.bigscreen")
 
 public:
-    explicit BigscreenPlugin(QObject* parent, const QVariantList& args);
-    ~BigscreenPlugin() override;
-
-    bool receivePacket(const NetworkPacket& np) override;
-    void connected() override {};
+    using KdeConnectPlugin::KdeConnectPlugin;
+    void receivePacket(const NetworkPacket &np) override;
     QString dbusPath() const override;
 
 Q_SIGNALS:
-    Q_SCRIPTABLE void messageReceived(const QString& message);
-
+    Q_SCRIPTABLE void messageReceived(const QString &message);
 };
-
-#endif
