@@ -5,10 +5,11 @@
  */
 
 #include "mpriscontrolplugin-win.h"
-#include "plugin_mpris_debug.h"
 
+#include "plugin_mpriscontrol_debug.h"
 #include <core/device.h>
 
+#include <KLocalizedString>
 #include <KPluginFactory>
 
 #include <QBuffer>
@@ -17,16 +18,23 @@
 #include <random>
 
 #include <ppltasks.h>
+#include <windows.h>
 
 using namespace Windows::Foundation;
 
 K_PLUGIN_CLASS_WITH_JSON(MprisControlPlugin, "kdeconnect_mpriscontrol.json")
 
+namespace
+{
+const QString DEFAULT_PLAYER =
+    i18nc("@title Users select this to control the current media player when we can't detect a specific player name like VLC", "Current Player");
+}
+
 MprisControlPlugin::MprisControlPlugin(QObject *parent, const QVariantList &args)
     : KdeConnectPlugin(parent, args)
     , sessionManager(GlobalSystemMediaTransportControlsSessionManager::RequestAsync().get())
 {
-    sessionManager->SessionsChanged([this](GlobalSystemMediaTransportControlsSessionManager, SessionsChangedEventArgs) {
+    sessionManager.SessionsChanged([this](GlobalSystemMediaTransportControlsSessionManager, SessionsChangedEventArgs) {
         this->updatePlayerList();
     });
     this->updatePlayerList();
@@ -37,7 +45,7 @@ std::optional<QString> MprisControlPlugin::getPlayerName(GlobalSystemMediaTransp
     auto entry = std::find(this->playerList.constBegin(), this->playerList.constEnd(), player);
 
     if (entry == this->playerList.constEnd()) {
-        qCWarning(KDECONNECT_PLUGIN_MPRIS) << "PlaybackInfoChanged received for no longer tracked session" << player.SourceAppUserModelId().c_str();
+        qCWarning(KDECONNECT_PLUGIN_MPRISCONTROL) << "PlaybackInfoChanged received for no longer tracked session" << player.SourceAppUserModelId().c_str();
         return std::nullopt;
     }
 
@@ -158,7 +166,7 @@ void MprisControlPlugin::updatePlayerList()
     mediaPropertiesChangedHandlers.clear();
     timelinePropertiesChangedHandlers.clear();
 
-    auto sessions = sessionManager->GetSessions();
+    auto sessions = sessionManager.GetSessions();
     playbackInfoChangedHandlers.resize(sessions.Size());
     mediaPropertiesChangedHandlers.resize(sessions.Size());
     timelinePropertiesChangedHandlers.resize(sessions.Size());
@@ -172,7 +180,7 @@ void MprisControlPlugin::updatePlayerList()
         try {
             playerName = AppInfo::GetFromAppUserModelId(playerName).DisplayInfo().DisplayName();
         } catch (winrt::hresult_error e) {
-            qCDebug(KDECONNECT_PLUGIN_MPRIS) << QString::fromWCharArray(playerName.c_str()) << "doesn\'t have a valid AppUserModelID! Sending as-is..";
+            qCDebug(KDECONNECT_PLUGIN_MPRISCONTROL) << QString::fromWCharArray(playerName.c_str()) << "doesn\'t have a valid AppUserModelID! Sending as-is..";
         }
 #endif
         QString uniqueName = QString::fromWCharArray(playerName.c_str());
@@ -238,7 +246,7 @@ void MprisControlPlugin::sendPlayerList()
 {
     NetworkPacket np(PACKET_TYPE_MPRIS);
 
-    np.set(QStringLiteral("playerList"), playerList.keys());
+    np.set(QStringLiteral("playerList"), playerList.keys() + QStringList(DEFAULT_PLAYER));
     np.set(QStringLiteral("supportAlbumArtPayload"), false); // TODO: Sending albumArt doesn't work
 
     sendPacket(np);
@@ -248,7 +256,7 @@ bool MprisControlPlugin::sendAlbumArt(std::variant<NetworkPacket, QString> const
                                       GlobalSystemMediaTransportControlsSession const &player,
                                       QString artUrl)
 {
-    qWarning(KDECONNECT_PLUGIN_MPRIS) << "Sending Album Art";
+    qWarning(KDECONNECT_PLUGIN_MPRISCONTROL) << "Sending Album Art";
     NetworkPacket np = packetOrName.index() == 0 ? std::get<0>(packetOrName) : NetworkPacket(PACKET_TYPE_MPRIS);
     if (packetOrName.index() == 1)
         np.set(QStringLiteral("player"), std::get<1>(packetOrName));
@@ -279,27 +287,92 @@ bool MprisControlPlugin::sendAlbumArt(std::variant<NetworkPacket, QString> const
     }
 }
 
-bool MprisControlPlugin::receivePacket(const NetworkPacket &np)
+void MprisControlPlugin::handleDefaultPlayer(const NetworkPacket &np)
+{
+    if (np.has(QStringLiteral("action"))) {
+        INPUT input = {0};
+        input.type = INPUT_KEYBOARD;
+
+        input.ki.time = 0;
+        input.ki.dwExtraInfo = 0;
+        input.ki.wScan = 0;
+        input.ki.dwFlags = 0;
+
+        const QString &action = np.get<QString>(QStringLiteral("action"));
+
+        if (action == QStringLiteral("PlayPause") || (action == QStringLiteral("Play")) || (action == QStringLiteral("Pause"))) {
+            input.ki.wVk = VK_MEDIA_PLAY_PAUSE;
+            ::SendInput(1, &input, sizeof(INPUT));
+        } else if (action == QStringLiteral("Stop")) {
+            input.ki.wVk = VK_MEDIA_STOP;
+            ::SendInput(1, &input, sizeof(INPUT));
+        } else if (action == QStringLiteral("Next")) {
+            input.ki.wVk = VK_MEDIA_NEXT_TRACK;
+            ::SendInput(1, &input, sizeof(INPUT));
+        } else if (action == QStringLiteral("Previous")) {
+            input.ki.wVk = VK_MEDIA_PREV_TRACK;
+            ::SendInput(1, &input, sizeof(INPUT));
+        } else if (action == QStringLiteral("Stop")) {
+            input.ki.wVk = VK_MEDIA_STOP;
+            ::SendInput(1, &input, sizeof(INPUT));
+        }
+    }
+
+    // Send something read from the mpris interface
+    NetworkPacket answer(PACKET_TYPE_MPRIS);
+    answer.set(QStringLiteral("player"), DEFAULT_PLAYER);
+    bool somethingToSend = false;
+    if (np.get<bool>(QStringLiteral("requestNowPlaying"))) {
+        answer.set(QStringLiteral("pos"), 0);
+        answer.set(QStringLiteral("isPlaying"), false);
+        answer.set(QStringLiteral("canPause"), false);
+        answer.set(QStringLiteral("canPlay"), true);
+        answer.set(QStringLiteral("canGoNext"), true);
+        answer.set(QStringLiteral("canGoPrevious"), true);
+        answer.set(QStringLiteral("canSeek"), false);
+        somethingToSend = true;
+    }
+
+    if (np.get<bool>(QStringLiteral("requestVolume"))) {
+        // we don't support setting per-app volume levels yet
+        answer.set(QStringLiteral("volume"), -1);
+        somethingToSend = true;
+    }
+
+    if (somethingToSend) {
+        sendPacket(answer);
+    }
+}
+
+void MprisControlPlugin::receivePacket(const NetworkPacket &np)
 {
     if (np.has(QStringLiteral("playerList"))) {
-        return false; // Whoever sent this is an mpris client and not an mpris control!
+        return; // Whoever sent this is an mpris client and not an mpris control!
     }
 
     // Send the player list
     const QString name = np.get<QString>(QStringLiteral("player"));
+
+    if (name == DEFAULT_PLAYER) {
+        handleDefaultPlayer(np);
+        return;
+    }
+
     auto it = playerList.find(name);
     bool valid_player = (it != playerList.end());
     if (!valid_player || np.get<bool>(QStringLiteral("requestPlayerList"))) {
+        updatePlayerList();
         sendPlayerList();
         if (!valid_player) {
-            return true;
+            return;
         }
     }
 
     auto player = it.value();
 
     if (np.has(QStringLiteral("albumArtUrl"))) {
-        return sendAlbumArt(name, player, np.get<QString>(QStringLiteral("albumArtUrl")));
+        sendAlbumArt(name, player, np.get<QString>(QStringLiteral("albumArtUrl")));
+        return;
     }
 
     if (np.has(QStringLiteral("action"))) {
@@ -319,11 +392,11 @@ bool MprisControlPlugin::receivePacket(const NetworkPacket &np)
         }
     }
     if (np.has(QStringLiteral("setVolume"))) {
-        qWarning(KDECONNECT_PLUGIN_MPRIS) << "Setting volume is not supported";
+        qWarning(KDECONNECT_PLUGIN_MPRISCONTROL) << "Setting volume is not supported";
     }
     if (np.has(QStringLiteral("Seek"))) {
         TimeSpan offset = std::chrono::microseconds(np.get<int>(QStringLiteral("Seek")));
-        qWarning(KDECONNECT_PLUGIN_MPRIS) << "Seeking" << offset.count() << "ns to" << name;
+        qWarning(KDECONNECT_PLUGIN_MPRISCONTROL) << "Seeking" << offset.count() << "ns to" << name;
         player.TryChangePlaybackPositionAsync((player.GetTimelineProperties().Position() + offset).count()).get();
     }
 
@@ -366,8 +439,7 @@ bool MprisControlPlugin::receivePacket(const NetworkPacket &np)
     if (somethingToSend) {
         sendPacket(answer);
     }
-
-    return true;
 }
 
+#include "moc_mpriscontrolplugin-win.cpp"
 #include "mpriscontrolplugin-win.moc"
