@@ -5,14 +5,11 @@
  */
 
 #include "notification.h"
-#include "plugin_notification_debug.h"
+#include "plugin_notifications_debug.h"
 
-#include "knotifications_version.h"
-#include <KNotification>
-#if KNOTIFICATIONS_VERSION >= QT_VERSION_CHECK(5, 81, 0)
-#include <KNotificationReplyAction>
-#endif
 #include <KLocalizedString>
+#include <KNotification>
+#include <KNotificationReplyAction>
 #include <QFile>
 #include <QIcon>
 #include <QJsonArray>
@@ -47,22 +44,6 @@ Notification::Notification(const NetworkPacket &np, const Device *device, QObjec
 
     parseNetworkPacket(np);
     createKNotification(np);
-
-    connect(m_notification, QOverload<unsigned int>::of(&KNotification::activated), this, [this](unsigned int actionIndex) {
-// Since 5.81 we use KNotification's inline reply instead of our own action
-#if KNOTIFICATIONS_VERSION < QT_VERSION_CHECK(5, 81, 0)
-        // Do nothing for our own reply action
-        if (!m_requestReplyId.isEmpty() && actionIndex == 1) {
-            return;
-        }
-#endif
-        // Notification action indices start at 1
-        Q_EMIT actionTriggered(m_internalId, m_actions[actionIndex - 1]);
-    });
-}
-
-Notification::~Notification()
-{
 }
 
 void Notification::dismiss()
@@ -124,20 +105,22 @@ void Notification::createKNotification(const NetworkPacket &np)
     m_notification->setHint(QStringLiteral("x-kde-origin-name"), m_device->name());
 
     if (!m_requestReplyId.isEmpty()) {
-#if KNOTIFICATIONS_VERSION >= QT_VERSION_CHECK(5, 81, 0)
         auto replyAction = std::make_unique<KNotificationReplyAction>(i18nc("@action:button", "Reply"));
         replyAction->setPlaceholderText(i18nc("@info:placeholder", "Reply to %1...", m_appName));
         replyAction->setFallbackBehavior(KNotificationReplyAction::FallbackBehavior::UseRegularAction);
         QObject::connect(replyAction.get(), &KNotificationReplyAction::replied, this, &Notification::replied);
         QObject::connect(replyAction.get(), &KNotificationReplyAction::activated, this, &Notification::reply);
         m_notification->setReplyAction(std::move(replyAction));
-#else
-        m_actions.prepend(i18n("Reply"));
-        connect(m_notification, &KNotification::action1Activated, this, &Notification::reply, Qt::UniqueConnection);
-#endif
     }
 
-    m_notification->setActions(m_actions);
+    m_notification->clearActions();
+    for (const QString &actionId : std::as_const(m_actions)) {
+        KNotificationAction *action = m_notification->addAction(actionId);
+
+        connect(action, &KNotificationAction::activated, this, [this, actionId] {
+            Q_EMIT actionTriggered(m_internalId, actionId);
+        });
+    }
 
     m_hasIcon = m_hasIcon && !m_payloadHash.isEmpty();
 
@@ -167,7 +150,7 @@ void Notification::loadIcon(const NetworkPacket &np)
         connect(fileTransferJob, &FileTransferJob::result, this, [this, fileTransferJob] {
             s_downloadsInProgress.remove(m_iconPath);
             if (fileTransferJob->error()) {
-                qCDebug(KDECONNECT_PLUGIN_NOTIFICATION) << "Error in FileTransferJob: " << fileTransferJob->errorString();
+                qCDebug(KDECONNECT_PLUGIN_NOTIFICATIONS) << "Error in FileTransferJob: " << fileTransferJob->errorString();
             } else {
                 applyIcon();
             }
@@ -212,3 +195,5 @@ void Notification::parseNetworkPacket(const NetworkPacket &np)
         m_actions.append(value.toString());
     }
 }
+
+#include "moc_notification.cpp"
