@@ -21,16 +21,7 @@
 
 K_PLUGIN_CLASS_WITH_JSON(RemoteSystemVolumePlugin, "kdeconnect_remotesystemvolume.json")
 
-RemoteSystemVolumePlugin::RemoteSystemVolumePlugin(QObject *parent, const QVariantList &args)
-    : KdeConnectPlugin(parent, args)
-{
-}
-
-RemoteSystemVolumePlugin::~RemoteSystemVolumePlugin()
-{
-}
-
-bool RemoteSystemVolumePlugin::receivePacket(const NetworkPacket &np)
+void RemoteSystemVolumePlugin::receivePacket(const NetworkPacket &np)
 {
     if (np.has(QStringLiteral("sinkList"))) {
         QJsonDocument document(np.get<QJsonArray>(QStringLiteral("sinkList")));
@@ -47,8 +38,6 @@ bool RemoteSystemVolumePlugin::receivePacket(const NetworkPacket &np)
             Q_EMIT mutedChanged(name, np.get<int>(QStringLiteral("muted")));
         }
     }
-
-    return true;
 }
 
 void RemoteSystemVolumePlugin::sendVolume(const QString &name, int volume)
@@ -81,7 +70,8 @@ QByteArray RemoteSystemVolumePlugin::sinks()
 
 QString RemoteSystemVolumePlugin::dbusPath() const
 {
-    return QStringLiteral("/modules/kdeconnect/devices/") + device()->id() + QStringLiteral("/remotesystemvolume");
+    return QLatin1String("/modules/kdeconnect/devices/%1/remotesystemvolume").arg(device()->id());
 }
 
+#include "moc_remotesystemvolumeplugin.cpp"
 #include "remotesystemvolumeplugin.moc"
