@@ -21,10 +21,6 @@ NotifyingApplicationModel::NotifyingApplicationModel(QObject *parent)
 {
 }
 
-NotifyingApplicationModel::~NotifyingApplicationModel()
-{
-}
-
 QVector<NotifyingApplication> NotifyingApplicationModel::apps()
 {
     return m_apps;
@@ -77,7 +73,6 @@ void NotifyingApplicationModel::clearApplications()
 
 QVariant NotifyingApplicationModel::data(const QModelIndex &index, int role) const
 {
-    Q_UNUSED(role);
     if (!index.isValid() || index.row() < 0 || index.row() >= m_apps.size() || index.column() > 3) {
         return QVariant();
     }
@@ -212,3 +207,5 @@ int NotifyingApplicationModel::rowCount(const QModelIndex &parent) const
     }
     return m_apps.size();
 }
+
+#include "moc_notifyingapplicationmodel.cpp"
